@@ -17,9 +17,11 @@
 #ifndef GLOBALS_H_INCLUDED
 #define GLOBALS_H_INCLUDED 1
 
+#include <stdarg.h>
 #include "defs.h"
 #include "deduper.h"
 #include "sort.h"
+#include "time.h"
 
 #ifdef MAIN_PROGRAM
 #define EXTERN
@@ -34,18 +36,20 @@ extern const struct verb verbs[];
 #endif
 
 EXTERN	const char id_swclient[]	INIT("dnsdbq");
-EXTERN	const char id_version[]		INIT("2.6.4");
+EXTERN	const char id_version[]		INIT("2.6.7");
 EXTERN	const char *program_name	INIT(NULL);
 EXTERN	const char path_sort[]		INIT("/usr/bin/sort");
 EXTERN	const char json_header[]	INIT("Accept: application/json");
 EXTERN	const char jsonl_header[]	INIT("Accept: application/x-ndjson");
 EXTERN	const char env_time_fmt[]	INIT("DNSDBQ_TIME_FORMAT");
 EXTERN	const char env_config_file[]	INIT("DNSDBQ_CONFIG_FILE");
+EXTERN	const char env_timeout[]	INIT("DNSDBQ_TIMEOUT");
 EXTERN	const char status_noerror[]	INIT("NOERROR");
 EXTERN	const char status_error[]	INIT("ERROR");
 EXTERN	const char *asinfo_domain	INIT("asn.routeviews.org");
 EXTERN	struct qparam qparam_empty INIT({ .query_limit = -1L,
 			.explicit_output_limit = -1L, .output_limit = -1L });
+EXTERN	char *cookie_file		INIT(NULL);
 EXTERN	char *config_file		INIT(NULL);
 EXTERN	verb_ct pverb			INIT(NULL);
 EXTERN	pdns_system_ct psys		INIT(NULL);
@@ -62,10 +66,11 @@ EXTERN	sort_e sorting			INIT(no_sort);
 EXTERN	batch_e batching		INIT(batch_none);
 EXTERN	present_e presentation		INIT(pres_none);
 EXTERN	char *presentation_name		INIT(NULL);
-EXTERN	present_t presenter		INIT(NULL);
+EXTERN	presenter_ct presenter		INIT(NULL);
 EXTERN	struct timeval startup_time	INIT({});
 EXTERN	int exit_code			INIT(0);
 EXTERN	long curl_ipresolve		INIT(CURL_IPRESOLVE_WHATEVER);
+EXTERN	long curl_timeout		INIT(0L);
 EXTERN	deduper_t minimal_deduper	INIT(NULL);
 
 /* deduplication table size. trades memory efficiency (an array of this many
@@ -80,4 +85,15 @@ EXTERN	const size_t minimal_modulus	INIT(10000);
 __attribute__((noreturn)) void my_exit(int);
 __attribute__((noreturn)) void my_panic(bool, const char *);
 
+/* my_logf -- annotate to stderr with program name and current time */
+static inline void
+my_logf(const char *fmtstr, ...) {
+	va_list ap;
+
+	va_start(ap, fmtstr);
+	fprintf(stderr, "%s [%s]: ", program_name, timeval_str(NULL, true));
+	vfprintf(stderr, fmtstr, ap);
+	putc('\n', stderr);
+}
+
 #endif /*GLOBALS_H_INCLUDED*/
