@@ -30,6 +30,7 @@
 #include "netio.h"
 #include "pdns.h"
 #include "globals.h"
+#include "time.h"
 
 static void io_drain(void);
 static void fetch_reap(fetch_t);
@@ -61,8 +62,7 @@ make_curl(void) {
 	curl_cleanup_needed = true;
 	multi = curl_multi_init();
 	if (multi == NULL) {
-		fprintf(stderr, "%s: curl_multi_init() failed\n",
-			program_name);
+		my_logf("curl_multi_init() failed");
 		my_exit(1);
 	}
 }
@@ -112,9 +112,20 @@ create_fetch(query_t query, char *url) {
 		curl_easy_setopt(fetch->easy,
 				 CURLOPT_IPRESOLVE, curl_ipresolve);
 
+	/* if user specified a timeout, use for connection and transactions. */
+	if (curl_timeout != 0L) {
+		curl_easy_setopt(fetch->easy,
+				 CURLOPT_CONNECTTIMEOUT, curl_timeout);
+		curl_easy_setopt(fetch->easy,
+				 CURLOPT_TIMEOUT, curl_timeout);
+	}
 	if (psys->auth != NULL)
 	    psys->auth(fetch);
 
+	/* if user specified a cookie file, tell libcurl about it. */
+	if (cookie_file != NULL)
+		curl_easy_setopt(fetch->easy, CURLOPT_COOKIEFILE, cookie_file);
+
 	if (psys->encap == encap_saf)
 		fetch->hdrs = curl_slist_append(fetch->hdrs, jsonl_header);
 	else
@@ -139,8 +150,8 @@ create_fetch(query_t query, char *url) {
 
 	res = curl_multi_add_handle(multi, fetch->easy);
 	if (res != CURLM_OK) {
-		fprintf(stderr, "%s: curl_multi_add_handle() failed: %s\n",
-			program_name, curl_multi_strerror(res));
+		my_logf("curl_multi_add_handle() failed: %s",
+			curl_multi_strerror(res));
 		my_exit(1);
 	}
 	return fetch;
@@ -265,7 +276,7 @@ writer_func(char *ptr, size_t size, size_t nmemb, void *blob) {
 	fetch_t fetch = (fetch_t) blob;
 	query_t query = fetch->query;
 	writer_t writer = query->writer;
-	qparam_ct qp = &query->params;
+	qparam_ct qp = &query->qp;
 	size_t bytes = size * nmemb;
 	char *nl;
 
@@ -280,17 +291,17 @@ writer_func(char *ptr, size_t size, size_t nmemb, void *blob) {
 				/* grab the token. */
 				writer->active = query;
 				DEBUG(2, true, "active (%d) %s\n",
-				      npaused, query->descrip);
+				      npaused, query->descr);
 			} else if (writer->active != query) {
 				/* pause the query. */
 				paused[npaused++] = query;
 				DEBUG(2, true, "pause (%d) %s\n",
-				      npaused, query->descrip);
+				      npaused, query->descr);
 				return CURL_WRITEFUNC_PAUSE;
 			}
 		}
 		if (!query->hdr_sent) {
-			printf("++ %s\n", query->descrip);
+			printf("++ %s\n", query->descr);
 			query->hdr_sent = true;
 		}
 	}
@@ -326,10 +337,8 @@ writer_func(char *ptr, size_t size, size_t nmemb, void *blob) {
 				curl_easy_getinfo(fetch->easy,
 						  CURLINFO_EFFECTIVE_URL,
 						  &url);
-				fprintf(stderr,
-					"%s: warning: libcurl %ld [%s] %s\n",
-					program_name, fetch->rcode,
-					url, message);
+				my_logf("warning: libcurl %ld [%s] %s",
+					fetch->rcode, url, message);
 			}
 			DESTROY(message);
 			fetch->buf[0] = '\0';
@@ -362,7 +371,7 @@ writer_func(char *ptr, size_t size, size_t nmemb, void *blob) {
 			       fetch->buf, pre_len + 1);
 			writer->ps_len += pre_len + 1;
 		} else {
-			query->writer->count += data_blob(fetch, pre_len);
+			query->writer->count += pdns_blob(fetch, pre_len);
 
 			if (psys->encap == encap_saf)
 				switch (fetch->saf_cond) {
@@ -375,8 +384,9 @@ writer_func(char *ptr, size_t size, size_t nmemb, void *blob) {
 				case sc_limited:
 				case sc_failed:
 				case sc_we_limited:
-					/* inform io_engine() intentional
-					 * abort. */
+					/* inform io_engine() that the abort
+					 * is intentional.
+					 */
 					fetch->stopped = true;
 					break;
 				}
@@ -396,7 +406,7 @@ last_fetch(fetch_t fetch) {
 
 	assert(query->fetches == fetch && fetch->next == NULL);
 	DEBUG(2, true, "query_done(%s), meta=%d\n",
-	      query->descrip, query->writer->meta_query);
+	      query->descr, query->writer->meta_query);
 	if (query->writer->meta_query)
 		return;
 
@@ -404,13 +414,13 @@ last_fetch(fetch_t fetch) {
 		const char *msg = or_else(fetch->saf_msg, "");
 
 		if (fetch->saf_cond == sc_limited)
-			fprintf(stderr, "Database limit: %s\n", msg);
+			my_logf("Database API limit: %s", msg);
 		else if (fetch->saf_cond == sc_failed)
-			fprintf(stderr, "Database result: %s\n", msg);
+			my_logf("Database result: %s", msg);
 		else if (fetch->saf_cond == sc_missing)
-			fprintf(stderr, "Query response missing: %s\n", msg);
+			my_logf("API response missing: %s", msg);
 		else if (query->status != NULL && !query->multitype)
-			fprintf(stderr, "Query status: %s (%s)\n",
+			my_logf("API status: %s (%s)",
 				query->status, query->message);
 	} else if (batching == batch_verbose) {
 		/* if this was an actively written query, unpause another. */
@@ -440,7 +450,7 @@ last_fetch(fetch_t fetch) {
 			     ufetch != NULL;
 			     ufetch = ufetch->next) {
 				DEBUG(2, true, "unpause (%d) %s\n",
-				      npaused, unpause->descrip);
+				      npaused, unpause->descr);
 				curl_easy_pause(ufetch->easy, CURLPAUSE_CONT);
 			}
 		}
@@ -480,9 +490,8 @@ writer_fini(writer_t writer) {
 			/* release any buffered info. */
 			DESTROY(fetch->buf);
 			if (fetch->len != 0) {
-				fprintf(stderr,
-					"%s: warning: stranding %d octets!\n",
-					program_name, (int)fetch->len);
+				my_logf("warning: stranding %d octets!",
+					(int)fetch->len);
 				fetch->len = 0;
 			}
 
@@ -497,7 +506,7 @@ writer_fini(writer_t writer) {
 		assert((query->status != NULL) == (query->message != NULL));
 		DESTROY(query->status);
 		DESTROY(query->message);
-		DESTROY(query->descrip);
+		DESTROY(query->descr);
 		DESTROY(query);
 		writer->queries = query_next;
 	}
@@ -537,9 +546,7 @@ writer_fini(writer_t writer) {
 			size_t len;
 
 			if ((nl = strchr(line, '\n')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no \\n found in '%s'\n",
-					program_name, line);
+				my_logf("warning: no \\n found in '%s'", line);
 				continue;
 			}
 			linep = line;
@@ -549,72 +556,43 @@ writer_fini(writer_t writer) {
 				 linep);
 			/* skip sort key: first */
 			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no SP found in '%s'\n",
-					program_name, line);
+				my_logf("warning: no SP found in '%s'", line);
 				continue;
 			}
 			linep += strspn(linep, " ");
 			/* skip sort key: last */
 			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no second SP in '%s'\n",
-					program_name, line);
+				my_logf("warning: no second SP in '%s'", line);
 				continue;
 			}
 			linep += strspn(linep, " ");
 			/* skip sort key: duration */
 			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no third SP in '%s'\n",
-					program_name, line);
+				my_logf("warning: no third SP in '%s'", line);
 				continue;
 			}
 			linep += strspn(linep, " ");
 			/* skip sort key: count */
 			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no fourth SP in '%s'\n",
-					program_name, line);
+				my_logf("warning: no fourth SP in '%s'", line);
 				continue;
 			}
 			linep += strspn(linep, " ");
 			/* skip sort key: rrname */
 			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no fifth SP in '%s'\n",
-					program_name, line);
+				my_logf("warning: no fifth SP in '%s'", line);
 				continue;
 			}
 			linep += strspn(linep, " ");
 			/* skip sort key: rrtype */
 			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no sixth SP in '%s'\n",
-					program_name, line);
+				my_logf("warning: no sixth SP in '%s'", line);
 				continue;
 			}
 			linep += strspn(linep, " ");
 			/* skip sort key: rdata */
 			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no seventh SP in '%s'\n",
-					program_name, line);
-				continue;
-			}
-			linep += strspn(linep, " ");
-			/* recover the mode */
-			mode_e mode = (mode_e) (int) strtol(linep, NULL, 10);
-			if (mode == no_mode) {
-				fprintf(stderr,
-					"%s: invalid mode from sort in '%s'\n",
-					program_name, line);
-				continue;
-			}
-			if ((linep = strchr(linep, ' ')) == NULL) {
-				fprintf(stderr,
-					"%s: warning: no eighth SP in '%s'\n",
-					program_name, line);
+				my_logf("warning: no seventh SP in '%s'", line);
 				continue;
 			}
 			linep += strspn(linep, " ");
@@ -626,12 +604,13 @@ writer_fini(writer_t writer) {
 			len = (size_t)(nl - linep);
 			msg = tuple_make(&tup, linep, len);
 			if (msg != NULL) {
-				fprintf(stderr,
-					"%s: warning: tuple_make: %s\n",
-					program_name, msg);
+				my_logf("warning: tuple_make: %s", msg);
 				continue;
 			}
-			(*presenter)(&tup, mode, writer);
+			/* after the sort, we don't know what query
+			 * caused any given tuple.
+			 */
+			(*presenter->output)(&tup, NULL, writer);
 			tuple_unmake(&tup);
 			count++;
 		}
@@ -643,10 +622,8 @@ writer_fini(writer_t writer) {
 			perror("waitpid");
 		} else {
 			if (!writer->sort_killed && status != 0)
-				fprintf(stderr,
-					"%s: warning: sort "
-					"exit status is %u\n",
-					program_name, (unsigned)status);
+				my_logf("warning: sort exit status is %u",
+					(unsigned)status);
 		}
 	}
 
@@ -730,7 +707,7 @@ io_drain(void) {
 						  &fetch->rcode);
 
 			DEBUG(2, true, "io_drain(%s) DONE rcode=%d\n",
-			      query->descrip, fetch->rcode);
+			      query->descr, fetch->rcode);
 			if (psys->encap == encap_saf) {
 				if (fetch->saf_cond == sc_begin ||
 				    fetch->saf_cond == sc_ongoing)
@@ -753,24 +730,19 @@ io_drain(void) {
 				      or_else(fetch->saf_msg, ""));
 			}
 			if (cm->data.result == CURLE_COULDNT_RESOLVE_HOST) {
-				fprintf(stderr,
-					"%s: warning: libcurl failed since "
-					"could not resolve host\n",
-					program_name);
+				my_logf("libcurl failed since "
+					"could not resolve host");
 				exit_code = 1;
 			} else if (cm->data.result == CURLE_COULDNT_CONNECT) {
-				fprintf(stderr,
-					"%s: warning: libcurl failed since "
-					"could not connect\n",
-					program_name);
+				my_logf("libcurl failed since "
+					"could not connect");
 				exit_code = 1;
 			} else if (cm->data.result != CURLE_OK &&
 				   !fetch->stopped)
 			{
-				fprintf(stderr,
-					"%s: warning: libcurl failed with "
-					"curl error %d (%s)\n",
-					program_name, cm->data.result,
+				my_logf("libcurl failed with "
+					"curl error %d (%s)",
+					cm->data.result,
 					curl_easy_strerror(cm->data.result));
 				exit_code = 1;
 			}
@@ -805,8 +777,7 @@ escape(const char *str) {
 		return NULL;
 	escaped = curl_escape(str, (int)strlen(str));
 	if (escaped == NULL) {
-		fprintf(stderr, "%s: curl_escape(%s) failed\n",
-			program_name, str);
+		my_logf("curl_escape(%s) failed", str);
 		my_exit(1);
 	}
 	ret = strdup(escaped);
